/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "BidimHistoDetectorElement.h"

namespace npp {

const string BidimHistoDetectorElement::TYPE = "npp_bidim_histo_detector_element";

/*
 * Constructor
 */
BidimHistoDetectorElement::BidimHistoDetectorElement(const string& name) :
	acquisition::DetectorElement(name) {

	registerFunction(TYPE);

	m_Driver.init(this, "driver");
}

/*
 * postConfiguration
 */
void BidimHistoDetectorElement::postConfiguration() {
	acquisition::DetectorElement::postConfiguration();
	registerUpdater(m_Driver->pf1bPsdBidimHistoZData, &BidimHistoDetectorElement::updateData, this);
}

/*
 * Destructor
 */
BidimHistoDetectorElement::~BidimHistoDetectorElement() {
}

/*
 * updateData
 */
void BidimHistoDetectorElement::updateData() {
	detSize = m_Driver->pf1bPsdBidimHistoZData.getSize();
	int32* ldata =  m_Driver->pf1bPsdBidimHistoZData();
	data.update(ldata);
	data.setSize(detSize());
	data.sendEvent();
	calculateDetectorSum();
}

}
